#ifndef __ATM8F22A4_H__
#define __ATM8F22A4_H__
#include <intrins.h>
typedef signed char int8_t;
typedef unsigned char uint8_t;
typedef signed short int int16_t;
typedef unsigned short int uint16_t;
typedef unsigned int uint_t;
typedef signed long int32_t;
typedef unsigned long uint32_t;

#define NOP() _nop_() // Keil 51  中的NOP命令

/* ------------------- BYTE Register-------------------- */
/* CPU */
sfr SP = 0x81;
sfr DPL = 0x82;
sfr DPH = 0x83;
sfr DPL1 = 0x84;
sfr DPH1 = 0x85;
sfr PCON = 0x87;
sfr DPS = 0xC9;
sfr DPC = 0xCA;
sfr PSW = 0xD0;
sfr ACC = 0xE0;
sfr B = 0xF0;
sfr BANK = 0xF8;

/*CLK*/
sfr CKCON = 0xD1;
sfr SYSCON = 0xD2;
sfr CLKCFG = 0xD3;
sfr CLKEN0 = 0xD4;
sfr CLKEN1 = 0xD5;
sfr BORCON = 0xD6;
sfr CLKSTAT = 0xD7;

/* FLASH */
sfr FSHCON = 0xC1;
sfr FSHDATL = 0xC2;
sfr FSHDATH = 0xC3;
sfr FSHADRL = 0xC4;
sfr FSHADRH = 0xC5;
sfr FSDIV = 0xC6;

/* INTERRUPT */
sfr IEN0 = 0xA8;
sfr IEN1 = 0xC0;
sfr IEN2 = 0xC8;
sfr IP00 = 0xB8;
sfr IP01 = 0xA5;
sfr IP10 = 0xA6;
sfr IP11 = 0xA7;
sfr IP20 = 0xCB;
sfr IP21 = 0xCC;

/* EXINT*/
sfr EINT0 = 0xB1;
sfr EINT1 = 0xB2;

/* PORT */
sfr P0 = 0x80;
sfr P0ODR = 0xD9;
sfr P0CR = 0xE1;
sfr P0PCR = 0xE9;
sfr P0SELL = 0xDD;
sfr P0SELH = 0xE5;
sfr P1 = 0x90;
sfr P1ODR = 0xDA;
sfr P1CR = 0xE2;
sfr P1PCR = 0xEA;
sfr P1SELL = 0xDE;
sfr P1SELH = 0xE6;
sfr P2 = 0xA0;
sfr P2ODR = 0xDB;
sfr P2CR = 0xE3;
sfr P2PCR = 0xEB;
sfr P2SELL = 0xDF;
sfr WPKEY = 0xF9;    ///*bank1*/
sfr IHRCTRIM = 0xFF; ///*bank1*/
sfr REMAP = 0xFA;    ///*bank1*/
sfr AEN0 = 0xEC;
sfr AEN1 = 0xE4;

/* TIMER 0/1 */
sfr TCON = 0x88;
sfr TMOD = 0x89;
sfr TL0 = 0x8A;
sfr TL1 = 0x8B;
sfr TH0 = 0x8C;
sfr TH1 = 0x8D;
sfr TIMPRS = 0x8E;

/* TIMER 2 */
sfr T2MOD = 0xB9;
sfr T2CPL1 = 0xBA;
sfr T2CPH1 = 0xBB;
sfr T2CPL2 = 0xBC;
sfr T2CPH2 = 0xBD;
sfr T2CAP = 0xBE;
sfr T2CAPS = 0xBF;
sfr T2CON = 0xB4;
sfr TL2 = 0xB5;
sfr TH2 = 0xB6;
sfr T2EXS = 0xB7;

/*PWM*/
sfr PWMCON = 0xA9;
sfr PWMCFG = 0xAA;
sfr PWMPERH = 0xAB;
sfr PWMPERL = 0xAC;
sfr PWMDR = 0xAD;
sfr PWMDF = 0xAE;
sfr EVTCON = 0xAF;
sfr CCR0H = 0x91;
sfr CCR0L = 0x92;
sfr CCR1H = 0x93;
sfr CCR1L = 0x94;
sfr CCR2H = 0x95;
sfr CCR2L = 0x96;
sfr PWMIE = 0x97;
sfr CCR3H = 0xF9;
sfr CCR3L = 0xFA;
sfr PWMOEN = 0xFB;
sfr PWMSEL0 = 0xFD;
sfr PWMSEL1 = 0xFC;
sfr PWMPOL = 0xED;
sfr CCIEN = 0xFE;
sfr CCISR = 0xFF;
sfr PWMLOCK = 0xEE;
sfr PWMFC = 0xEF;

/*UART */
sfr S0CON = 0x98;
sfr S0BUF = 0x99;
sfr S0RELL = 0x9A;
sfr S0RELH = 0x9B;
sfr S1CON = 0xA1;
sfr S1BUF = 0xA2;
sfr S1RELL = 0xA3;
sfr S1RELH = 0xA4;

/*BUZCON */
sfr BUZCON = 0xDC;

/*CPL */
sfr CPLCON = 0xE1;
sfr CPLCH0 = 0xE2;
sfr CPLCH1 = 0xE3;
sfr CPLCH2 = 0xE4;
sfr CPLCH3 = 0xE5;

/*ADC*/
sfr ADCON = 0xF1;
sfr ADCDATAH = 0xF2;
sfr ADCDATAL = 0xF3;
sfr ADCFG = 0xF4;
sfr ADTRG = 0xF5;
sfr ADREF = 0xF6;
sfr ADSAMP = 0xF7;

/*WDT*/
sfr WDTRLD = 0x86;
sfr WDTCON = 0xCE;
sfr WDTWK = 0xCF;

/*WT*/
sfr WTCON = 0xCD;

/*CRC*/
sfr CRCCON = 0x9C;
sfr CRCDAT = 0x9D;
sfr CRCSD0 = 0x9E;
sfr CRCSD1 = 0x9F;

/*REST*/
sfr SRST = 0x8F;  /*bank0*/
sfr RSTID = 0xC7; /*bank0*/
sfr SPOVL = 0xE6; /*bank1*/
sfr SPOVH = 0xE7; /*bank1*/

/*--------------------------  BIT Register -------------------- */
/*  PSW   */
sbit CY = 0xD7;
sbit AC = 0xD6;
sbit F0 = 0xD5;
sbit RS1 = 0xD4;
sbit RS0 = 0xD3;
sbit OV = 0xD2;
sbit F1 = 0XD1;
sbit P = 0xD0;
/*  ACC  */
sbit ACC_0 = ACC ^ 0;
sbit ACC_1 = ACC ^ 1;
sbit ACC_2 = ACC ^ 2;
sbit ACC_3 = ACC ^ 3;
sbit ACC_4 = ACC ^ 4;
sbit ACC_5 = ACC ^ 5;
sbit ACC_6 = ACC ^ 6;
sbit ACC_7 = ACC ^ 7;

/*   B  */
sbit B_0 = B ^ 0;
sbit B_1 = B ^ 1;
sbit B_2 = B ^ 2;
sbit B_3 = B ^ 3;
sbit B_4 = B ^ 4;
sbit B_5 = B ^ 5;
sbit B_6 = B ^ 6;
sbit B_7 = B ^ 7;

/*  TCON  */
sbit TF1 = 0x8F;
sbit TR1 = 0x8E;
sbit TF0 = 0x8D;
sbit TR0 = 0x8C;
sbit IE1 = 0x8B;
sbit IT1 = 0x8A;
sbit IE0 = 0x89;
sbit IT0 = 0x88;

/*  IEN0   */
sbit EA = 0xAF;
sbit ES1 = 0xAD;
sbit ES0 = 0xAC;
sbit ET1 = 0xAB;
sbit EX1 = 0xAA;
sbit ET0 = 0xA9;
sbit EX0 = 0xA8;

//---------------------------定义端口↓
sbit P0_0 = P0 ^ 0;
sbit P0_1 = P0 ^ 1;
sbit P0_2 = P0 ^ 2;
sbit P0_3 = P0 ^ 3;
sbit P0_4 = P0 ^ 4;
sbit P0_5 = P0 ^ 5;
sbit P0_6 = P0 ^ 6;
sbit P0_7 = P0 ^ 7;

sbit P1_0 = P1 ^ 0;
sbit P1_1 = P1 ^ 1;
sbit P1_2 = P1 ^ 2;
sbit P1_3 = P1 ^ 3;
sbit P1_4 = P1 ^ 4;
sbit P1_5 = P1 ^ 5;
sbit P1_6 = P1 ^ 6;
sbit P1_7 = P1 ^ 7;

sbit P2_0 = P2 ^ 0;
sbit P2_1 = P2 ^ 1;

#endif
