#define EEPROM_EXT
#include "Include.h"

uint16_t Flash_ReadDoubleChar(uint16_t addr)
{
    uint16_t temp;
    temp = *((uint16_t code *)(addr));

    return temp;
}
/*=========================================================================================\n
* @描述: 写入双字节
===========================================================================================*/
void Flash_WirteDoubleChar(uint16_t addr, uint16_t dat)
{
    uint8_t tempEA;
    FSHADRH = addr >> 8;
    FSHADRL = addr; // 设置起始地址0x3F00
    FSHDATH = dat >> 8;
    FSHDATL = dat;
    tempEA = EA;
    EA = 0;
    FSHCON = 0xA5;
    while (FSHCON & 0x01)
        ;
    EA = tempEA;
}

/*=========================================================================================\n
* @描述: 写入一个字节
===========================================================================================*/
void Flash_WirteChar(uint16_t addr, uint8_t dat)
{
    uint16_t temp = 0;

    temp = Flash_ReadDoubleChar(addr - (addr % 2)); // 把双字节都读出来 0x 低 高

    if (addr % 2 == 1) // 保留低字节，只写入高字节
    {
        temp = temp >> 8;
        temp += ((uint16_t)dat) << 8;
    }
    else // 保留高字节，只写入低字节
    {
        temp = temp << 8;
        temp += dat;
    }

    Flash_WirteDoubleChar(addr - (addr % 2), temp); // 按照大端写入
}

/*=========================================================================================\n
* @描述: 写FALSH数据,地址单数的先读取前一个字节，再写入
===========================================================================================*/

void WriteData(uint16_t addr, uint16_t len, uint8_t *dat) // Write wdata into flash
{
    uint16_t temp;
    temp = 0;

    if (addr % 2 == 1) // 地址从单数开始
    {
        Flash_WirteChar(addr, *dat);
        addr++;
        dat++;
        len--;
    }

    while (len >= 2)
    {
        temp = *(dat + 1) << 8;
        temp = temp + *dat;
        Flash_WirteDoubleChar(addr, temp); // 按照大端写入
        addr += 2;
        dat += 2;
        len -= 2;
        CLRWDT();
    }

    if (len == 1)
    {
        Flash_WirteChar(addr, *dat);
    }
}
