// #define __MAIN__
#define VAREXT
#include "Include.h"

const uint8_t code UPDATA_DEFAULT_TABLE[FLASH_RUN_TYPE_LEN] = {0x00, 0x00, 0x00, 0x00};

void CLK_Init(void)
{
    // SYSCON = Syscon_HRC_16M; //16MHZ-RC  8MHZ-CPU
    SYSCON &= 0x3F;
    SYSCON |= 0x80;
}

void Init_Timer0(void)
{
    TIMPRS = 0x07; // 256分频
    TCON &= 0xCF;  // 清标志位和启动位
    TMOD &= 0xf0;
    TMOD |= 0x02;

    TH0 = 0x82;
    TL0 = 0x82;

    TR0 = 1;
    ET0 = 1;
}

void GPIO_Init(void)
{
    P0PCR = 0x00;
    P0CR = 0x00;
    P0 = 0x00;
    P0SELL = 0x00;
    P0SELH = 0x00;

    P1PCR = 0x00;
    P1CR = 0x00;
    P1 = 0x00;
    P1SELL = 0x00;
#ifndef Debug
    P1SELH = 0x00;
#else
    P1SELH = 0x40;
#endif

    P2PCR = 0x00;
    P2CR = 0x00;
    P2 = 0x00;

#ifndef Debug
    P2SELL = 0x00;
#else
    P2SELL = 0x04;
#endif
    TEST_IOOUT();
}

void WDT_Init()
{
    WDTCON = 0x20; // 1024MS
    WDTCON = 0x22;
}

void Init_Var(void)
{
    guc_TimeEvent = 0;
    guc_2mscnt = 0;
    guc_250mscnt = 0;
    guc_secscnt = 0;

    guc_IapCommand = 0;
    guc_ComEvent = 0;
}

void CPUInit(void)
{
    EA = 0;
    CLK_Init();
    GPIO_Init();
    WDT_Init();
    CLRWDT();
    Init_Var();
    KeyInit();
    Init_Uart();
    ComBom_InitAll();
    Init_Timer0();
    EA = 1;
}

void Data_2msProc()
{
    KeyScan();
    if (guc_IapCommand)
    {
        if (gs_ComGroup[0].ucPort == Port_Idle)
        {
            Delay_1ms(2);
            guc_IapCommand = 0;
#pragma asm
            LJMP 0000H
#pragma endasm
                while (1);
        }
    }

    if (guc_2mscnt < 125)
    {
        ++guc_2mscnt;
        if (guc_2mscnt == 125)
        {
            guc_2mscnt = 0;
            guc_TimeEvent |= flgEtTim_250ms;
        }
    }
    else
    {
        guc_2mscnt = 0;
    }
    CLRWDT();
}

void Data_250msProc(void)
{
    CLRWDT();
    TEST_IOFALSH()
    if (guc_250mscnt < 4)
    {
        ++guc_250mscnt;
        if (guc_250mscnt == 4)
        {
            guc_250mscnt = 0;
            guc_TimeEvent |= flgEtTim_Second;
        }
    }
    else
    {
        guc_250mscnt = 0;
    }
}

void TimeEventProc(void)
{
    if (guc_TimeEvent & flgEtTim_2ms)
    {
        Data_2msProc();
        guc_TimeEvent &= ~flgEtTim_2ms;
    }
    else if (guc_TimeEvent & flgEtTim_250ms)
    {
        Data_250msProc();
        guc_TimeEvent &= ~flgEtTim_250ms;
    }
    else if (guc_TimeEvent & flgEtTim_Second)
    {
        guc_TimeEvent &= ~flgEtTim_Second;
    }
    else
    {
        guc_TimeEvent = 0;
    }
}

void IO_Test(void)
{
    P0CR |= 0x04;
    P0 |= 0x04;
    // Delay_1ms(30);
    // P0 &= 0xfb;
    // Delay_1ms(30);
}

void main(void)
{
    CPUInit();

    while (1)
    {
        IO_Test(); // 不要在循环里加入延时影响升级
        if (0x00 != guc_KeyEvent)
        {
            KeyProc();
        }
        if (0x00 != guc_TimeEvent)
        {
            TimeEventProc();
        }
        if (guc_ComEvent)
        {
            if (guc_ComEvent & flgCom_Complete)
            {
                ComApp_Proc();
                guc_ComEvent &= ~flgCom_Complete;
            }
            else
            {
                guc_ComEvent = 0;
            }
        }
    }
}
