#define COMBOTTOMEXT
#include "Include.h"
#ifdef UARTEN

extern const uint8_t code UPDATA_DEFAULT_TABLE[FLASH_RUN_TYPE_LEN];
uint8 guc_DyUart1Over; //
/**
 * @Fuction Name: uart0_init(void)
 * @Description: 串口1初始化
 * @param {type}
 * @return:
 */
// 系统时钟32/8=4MHz
void Uart1_Init(void)
{
    P1CR &= ~BIT7;
    P1PCR |= BIT7;
    P1SELH &= 0x3F;
    P1SELH |= 0xC0; // P17 -RXD

    P2CR |= BIT1;
    P2ODR &= ~BIT1;
    P2SELL = 0x0C; // P21 -tXD

    // 配置波特率
    S1RELH = 0x03;
    S1RELL = 0xF3;         // 38400
    BUZCON |= BIT1 + BIT2; // 自身波特率 加倍

    // 使用uart1模式1
    S1CON = 0x50; // 0x50;
    IEN0 |= BIT5;
}
void Uart0_Init(void)
{
    // 配置IO口 TX -> P0.3 RX -> P0.4
    P0CR |= BIT3;

    P0CR &= ~BIT4;
    P0PCR |= BIT4;

    P0SELH |= 0x03; // 选择复用功能
    P0SELL |= 0xC0;

    // 使用自身波特率发生器 波特率2*4Mhz/64/13=9615 ≈ 9600
    // BAUD = (2^SMOD)*FCLK/(64*(2^10 - S0REL))
    // FCLK为系统时钟
    S0RELH = 0x03;
    S0RELL = 0xF3;
    PCON |= 0xC0; // 使用自身波特率发生器

    // 使用uart0 模式1
    S0CON = 0x50;
    IE_UART0_enable;
}

void Init_Uart()
{
#if (UART == UART0)
    Uart0_Init();
#else
    Uart1_Init();
#endif
}
/*=========================================================================================\n
* @描述: 模拟串口的超时保护
*       为防止锁死，定时保护，当没有任何数据发送和接收的时候，初始化
===========================================================================================*/
void Uart_Dy10ms(void)
{
    guc_DyUart1Over--;

    if (0x00 == guc_DyUart1Over)
    {
        guc_DyUart1Over = Const_DyUart1Over; // 端口超时保护
        Init_Uart();                         // 初始化模拟串口
        ComBom_Init(ComIndex_Uart1);         // 将uart4 端口对应的 COM初始化
    }
}
/*=========================================================================================\n
* @描述: UART3接收,在中断中调用该函数
===========================================================================================*/
void Uart_Receive(void)
{
    //  guc_DyUart1Over = Const_DyUart1Over;//端口超时保护
    // 这里可以做奇偶校验判断
    // 处于空闲状态或已经处于uart接收状态
    //    if (SCON_USER & BIT0)//查询接收标志位
    //    {
    //        SCON_USER &= ~(BIT0);
    gs_ComGroup[ComIndex_Uart1].ucPort = Port_Uart1;
    guc_comdelay = 3;
    gs_ComGroup[ComIndex_Uart1].ucRecvTmr = Const_DyRvPack; // 设置数据包接收超时时间

    if ((gs_ComGroup[ComIndex_Uart1].ucStt == ComStt_Idle) // 这个判断是防止发送的时候有接收中断进入的可能
        || (gs_ComGroup[ComIndex_Uart1].ucStt == ComStt_Recv))
    {
        if (gs_ComGroup[ComIndex_Uart1].ucLen < Const_MaxBufLen) // 判断 com中的buf是否溢出
        {
            guc_DyUart1Over = Const_DyUart1Over;                                                // 端口超时保护
            gs_ComGroup[ComIndex_Uart1].ucBuf[gs_ComGroup[ComIndex_Uart1].ucLen++] = SBUF_USER; // 数据存入缓冲区，指针加加
        }
    }
}
/*=========================================================================================\n
* @描述: UART3发送,中断中调用
===========================================================================================*/
void Uart_Transmit(void)
{
    guc_DyUart1Over = Const_DyUart1Over; // 端口超时保护

    if (gs_ComGroup[ComIndex_Uart1].ucPort == Port_Uart1)
    {
        // 处于空闲状态或已经处于uart接收状态
        if (gs_ComGroup[ComIndex_Uart1].ucStt == ComStt_Send)
        {
            if (gs_ComGroup[ComIndex_Uart1].ucPos >= gs_ComGroup[ComIndex_Uart1].ucLen)
            {
                Uart_RevEn();
                // 发送完毕,初始化通讯控制状态
                ComBom_Init(ComIndex_Uart1); // 将uart2 端口对应的 COM初始化
                Init_Uart();                 // 初始化模拟串口
            }
            else
            {
                // //这里可以做奇偶校验运算
                // ACC = gs_ComGroup[ComIndex_Uart1].ucBuf[gs_ComGroup[ComIndex_Uart1].ucPos]; //计算校验位
                // if(P==0)                                        //校验位
                // {
                //     S1CON&=(~BIT3);
                // }
                // else
                // {
                //     S1CON|=(BIT3);
                // }
                SBUF_USER = gs_ComGroup[ComIndex_Uart1].ucBuf[gs_ComGroup[ComIndex_Uart1].ucPos++];
            }
        }
        else
        {                                // 如果在发送中断标记中，进入但是状态不正确，则初始化
            ComBom_Init(ComIndex_Uart1); // 将uart3 端口对应的 COM初始化
            Init_Uart();                 // 初始化模拟串口
            Uart_RevEn();
        }
    }
}
/*=========================================================================================\n
* @描述: 指定COM口进行初始化
===========================================================================================*/
void ComBom_Init(uint8 ucNo)
{
    // 清空接收缓存
    if (ucNo < Const_MaxComNum) // 增加长度保护限制
    {
        gs_ComGroup[ucNo].ucPort = Port_Idle;
        gs_ComGroup[ucNo].ucLen = 0;
        gs_ComGroup[ucNo].ucPos = 0;
        //        gs_ComGroup[ucNo].ucFrmHeadCnt=0;
        // 初始化通讯状态
        gs_ComGroup[ucNo].ucStt = ComStt_Idle;
        gs_ComGroup[ucNo].ucRecvTmr = 0; // 定时器无效状态
        gs_ComGroup[ucNo].ucOverTmr = 0; // 定时器无效状态
    }
}

/*=========================================================================================\n
* @描述: 把所有的COM都初始化
===========================================================================================*/
void ComBom_InitAll(void)
{
    uint8 i;

    for (i = 0; i < Const_MaxComNum; i++)
    {
        ComBom_Init(i);
    }
}
/*=========================================================================================\n
* @描述: 根据入口参数，确定启动哪个端口的发送和把那个buff对应的数据发送出去
===========================================================================================*/
void ComBom_StartSend(uint8 ucNo)
{
    if (gs_ComGroup[ucNo].ucPort == Port_Uart1)
    {
        Uart_Transmit(); // uart6 启动发送
    }
    else
    {
        ComBom_Init(ucNo); // 无效，初始化COM
    }
}
/*=========================================================================================\n
* @描述: 根据入口参数，实现不同的延时，提供给1ms中断调用
*       通过该实现是否一个数据包结束标志
===========================================================================================*/
void ComBom_10msDy(void)
{
    uint8 i;
    for (i = 0; i < Const_MaxComNum; i++)
    {                                     // 对COM进行延时
        if (gs_ComGroup[i].ucRecvTmr > 0) // 判断接收超时
        {
            gs_ComGroup[i].ucRecvTmr--;
            if (0 == gs_ComGroup[i].ucRecvTmr) // 如果间隔一定时间内未收到有效数据，则表示一个数据包已经收到
            {
                gs_ComGroup[i].ucStt = ComStt_WaitProc; // 处于等待处理模式，提交给应用层
                guc_ComEvent |= flgCom_Complete;
            }
        }
    }
}

/*=========================================================================================\n
* @描述: 通讯处理函数，主程序调用
===========================================================================================*/
void ComApp_Proc(void)
{
    uint8 port = 0;
    if ((gs_ComGroup[port].ucBuf[0] == 0x01) && (gs_ComGroup[port].ucBuf[1] == 0x55) && (gs_ComGroup[port].ucBuf[2] == 0xAA))
    {
        guc_IapCommand = 1;
        // WriteData(USER_FLASH_DATA_PAGE_ADDRESS, FLASH_RUN_TYPE_LEN, UPDATA_DEFAULT_TABLE); // 擦除APP 程序完好标志
        gs_ComGroup[port].ucStt = ComStt_Send;
        gs_ComGroup[port].ucBuf[0] = 0x02;
        gs_ComGroup[port].ucBuf[1] = 0xAA;
        gs_ComGroup[port].ucBuf[2] = 0x55;

        gs_ComGroup[port].ucLen = 3;
        gs_ComGroup[port].ucPos = 0;

        ComBom_StartSend(port);
    }
    else
    {
        ComBom_Init(port);
    }
}

#endif
